package vank_go

import (
	//"errors"
	"fmt"
	//"io"
	"io/ioutil"
	"os"
	"path/filepath"

	//"runtime"
	"testing"
)

/**
 *
 * This test demonstrates how supported document parsers treat metadata.
 *
 * when running this test, comment out statement removing temp folder:
 *
 *      defer os.RemoveAll(dir) // clean up
 *
 * and use a command similar to the following to review the document
 * metadata for each of the 14 documents indexed:
 *
 *      dumpindex  /tmp/test-repo-root129118166/index-1/parser_test.go-blog/w1565544533/ s
 *      dumpindex  /tmp/test-repo-root129118166/index-1/parser_test.go-blog/w1565544533/ dd 1 |less
 *
 *
**/
func TestAddFileClassType(t *testing.T) {
	//var err error = nil
	err := testAddFileClassType()
	if err != nil {
		//pc, fn, line, _ := runtime.Caller(2)
		//err = fmt.Errorf("[error] in %s[%s:%d] %v", runtime.FuncForPC(pc).Name(), fn, line, err)
		t.Fatal(err)
	}
}

//
// start of test logic implimentations
//

/*

   for supported file types, reference:
       https://sourceforge.net/p/lemur/wiki/Indexer%20File%20Formats/
   for preconfigured file class environment and parsers, review vank sources:
       FileClassEnvironmentFactory.cpp
       DocumentIteratorFactory.cpp
       TaggedDocumentIterator.cpp
       PDFDocumentExtractor.cpp
       PowerPointDocumentExtractor.cpp
       MboxDocumentIterator.cpp

   document indexing using the addFile and addString interfaces exhibit
   the following metadata extraction behavior.

   the metadata fields for an index is defined by the parameters supplied
   when the index is created. the only metadata requires is docno, used to
   prevent indexing duplicate documents. docno also represents the external
   name or id of the document.

   as of this writing, for a document being indexed, the parser always adds
   the following metadata for html, xml, text, pdf files. for example:

       path: /home/tavit/Work/dapp/gospace/src/github.com/dms3-fs/go-idx-vank/data/sitemap.xml
       docno: /home/tavit/Work/dapp/gospace/src/github.com/dms3-fs/go-idx-vank/data/sitemap.xml

   for html, xml, text documents, the parser additionally adds the metadata:

       filetype: TEXT

   for html, pdf documents, the parser additionally adds the metadata:

       url: /

   for pdf documents, the parser additionally adds the metadata:

       title: timeoutcdf.eps
       author: ongardie

   the parser additionally adds the metadata listed in a document, regardless
   of position in the document tag hierarchy (i.e. whether the metadata
   fields are wrapped within a <meta> tag). the metadata associated with a
   document is not required to include all the metadata keys specified during
   index creation.

   a file may contain multiple documents wrapped within a <document> tag, in
   the absence of such a tag, the file represents a single document.

   the indexer requires one metadata field for documents: docno, the value
   of this field must be unique within the index, otherwise the document is
   parsed but not indexed.

   when an application generated (using the addString interface) document
   includes a docno field, the index will contain two docno fields. the
   first added by the parser, and the second added by the application. the
   parser will use the last occurence as the authoritative docno record.

*/

func testAddFileClassType() (err error) {

	defer catch(&err)

	dir, err := ioutil.TempDir("", "test-repo-root")
	if err != nil {
		err = fmt.Errorf("failed to create TempDir: %v", err)
		return
	}
	defer os.RemoveAll(dir) // clean up

	var repositoryPath = filepath.Join(dir, "index-1")
	var corpusPath = filepath.Join(dir, "corpus-1")
	var corpusFileClass = "html"

	err = os.Mkdir(repositoryPath, 0775)
	if err != nil {
		err = fmt.Errorf("failed to create repository dir: %v", err)
		return
	}

	err = os.Mkdir(corpusPath, 0775)
	if err != nil {
		err = fmt.Errorf("failed to create corpus dir: %v", err)
		return
	}

	var is InfoSpace

	var a_app string = "parser_test.go"
	var a_space string = "blog"
	var a_server string = ""
	var a_command string = "VankBuildIndex"
	var a_index_path string = repositoryPath
	var a_corpus_path string = corpusPath
	var a_corpus_class string = corpusFileClass
	var a_build_param string = "data/params.xml"
	var a_stop_param string = "data/smallstop_param"
	is, err = NewInfoSpace(a_app,
		a_space,
		a_server,
		a_command,
		a_index_path,
		a_corpus_path,
		a_corpus_class,
		a_build_param,
		a_stop_param)
	if err != nil {
		return
	}
	defer DeleteInfoSpace(is)

	_, err = is.InfoSpaceAttach()
	if err != nil {
		return
	}

	///////////////////////////////////////////////////////////////////////////
	// now test adding document to the index
	///////////////////////////////////////////////////////////////////////////

	// add document as file, file extension implies corpusFileClass.
	// addFile should return a docId of -1
	// because include/vank/IndexEnvironment.hpp declares addFile() as void
	// however, document is added to the index

	//fmt.Printf("testAddFileClassType: add document as file, should return a docId of -1\n")
	fp, err := filepath.Abs("data/txdoc1.html")
	if err != nil {
		return
	}

	r_docId, err := is.InfoSpaceAddFile(fp)
	if DocIDToInt(r_docId) != -1 || err != nil {
		err = fmt.Errorf("testAddFileClassType: with InfoSpaceAddFile r_docId %v err %v\n", DocIDToInt(r_docId), err)
		return
	}

	// add document as string
	var docno1 string = `    <docno>offer-uidf5a8b61dacd606099e941250f695b21buid-f32d3adba31e486dc1ead5978ef6d7d39edb1393a2fe04d679b0adbb790d80140c9b7fbbe2081342c8157613dc56a80527bfb029bc42ac3b3d6e3dc7250ddb1f</docno>
`
	var docno2 string = `    <docno>offer-uidf5a8b61dacd606099e941250f695b22buid-f32d3adba31e486dc1ead5978ef6d7d39edb1393a2fe04d679b0adbb790d80140c9b7fbbe2081342c8157613dc56a80527bfb029bc42ac3b3d6e3dc7250ddb1f</docno>
`
	var common_preamble = `
	<document>
	  <meta>
	    <odmver>1</odmver>
	    <schver>1</schver>
	    <base-time>2016-09-24 14:42:51 UTC</base-time>
	    <curr-epoch>B2011</curr-epoch>
	    <prev-epoch>A2010</prev-epoch>
	    <max-areas>63</max-areas>
	    <max-cats>63</max-cats>
	    <app>vectavi</app>
`
	var common_rest = `
	<docver>f32d3adba31e486dc1ead5978ef6d7d39edb1393a2fe04d679b0adbb790d80140c9b7fbbe2081342c8157613dc56a80527bfb029bc42ac3b3d6e3dc7250ddb1f</docver>
  </meta>
  <tx>
    <tx-tim>
      <tx-tim-c1>0</tx-tim-c1>
      <tx-tim-c2>1050</tx-tim-c2>
      <tx-tim-c3>18</tx-tim-c3>
    </tx-tim>
    <tx-uid>uidf5a8b61dacd606099e941250f695b21buid</tx-uid>
    <tx-ipa>50.241.107.77</tx-ipa>
    <tx-cls>offer</tx-cls>
    <tx-sts>active</tx-sts>
    <tx-prv>secret</tx-prv>
    <tx-obj>badfood</tx-obj>
    <tx-sty>declared</tx-sty>
    <tx-src>anonymous</tx-src>
    <tx-brn>badfood</tx-brn>
    <tx-ttl>badfood</tx-ttl>
    <tx-adr>badfood</tx-adr>
    <tx-lat>
      <tx-lat-c1>0</tx-lat-c1>
      <tx-lat-c2>0</tx-lat-c2>
      <tx-lat-c3>0</tx-lat-c3>
      <tx-lat-c4>0</tx-lat-c4>
    </tx-lat>
    <tx-lon>
      <tx-lon-c1>0</tx-lon-c1>
      <tx-lon-c2>0</tx-lon-c2>
      <tx-lon-c3>0</tx-lon-c3>
      <tx-lon-c4>0</tx-lon-c4>
    </tx-lon>
    <tx-ops>
      <tx-ops-nam-cls>topic</tx-ops-nam-cls>
      <tx-ops-nam-top-des>verifying media upload with beta deployment </tx-ops-nam-top-des>
      <tx-ops-nam-top-cat>issue</tx-ops-nam-top-cat>
      <tx-ops-nam-top-med>
        <tx-ops-nam-top-med-cls>photo</tx-ops-nam-top-med-cls>
        <tx-ops-nam-top-med-len>1</tx-ops-nam-top-med-len>
        <tx-ops-nam-top-med-pth>/home/tavit/Work/repo/media/z9QYGirj3nb3v_-qdE0_8g.png</tx-ops-nam-top-med-pth>
        <tx-ops-nam-top-med-cls>video</tx-ops-nam-top-med-cls>
        <tx-ops-nam-top-med-tim>
          <tx-ops-nam-top-med-tim-c1>0</tx-ops-nam-top-med-tim-c1>
          <tx-ops-nam-top-med-tim-c2>0</tx-ops-nam-top-med-tim-c2>
        </tx-ops-nam-top-med-tim>
        <tx-ops-nam-top-med-pth>/home/tavit/Work/repo/media/zigmGfyL3l81qQZ77MkC_Q.MOV</tx-ops-nam-top-med-pth>
      </tx-ops-nam-top-med>
      <tx-ops-nam-top-sbs>
        <tx-ops-nam-top-sbs-cls>badfood</tx-ops-nam-top-sbs-cls>
        <tx-ops-nam-top-sbs-nam>test</tx-ops-nam-top-sbs-nam>
      </tx-ops-nam-top-sbs>
      <tx-ops-nam-cls>time</tx-ops-nam-cls>
      <tx-ops-nam-tim-lif>
        <tx-ops-nam-tim-lif-c1>0</tx-ops-nam-tim-lif-c1>
        <tx-ops-nam-tim-lif-c2>15</tx-ops-nam-tim-lif-c2>
        <tx-ops-nam-tim-lif-c3>0</tx-ops-nam-tim-lif-c3>
      </tx-ops-nam-tim-lif>
      <tx-ops-nam-cls>price</tx-ops-nam-cls>
      <tx-ops-nam-prc-xch>offer</tx-ops-nam-prc-xch>
      <tx-ops-nam-prc-por>
        <tx-ops-nam-prc-por-c1>0</tx-ops-nam-prc-por-c1>
        <tx-ops-nam-prc-por-c2>0</tx-ops-nam-prc-por-c2>
        <tx-ops-nam-prc-por-c3>0</tx-ops-nam-prc-por-c3>
        <tx-ops-nam-prc-por-c4>0</tx-ops-nam-prc-por-c4>
      </tx-ops-nam-prc-por>
      <tx-ops-nam-cls>place</tx-ops-nam-cls>
      <tx-ops-nam-plc-via>member</tx-ops-nam-plc-via>
      <tx-ops-nam-plc-adr>cartwright lane  billerica  middlesex county  massachusetts  01821  united states of america</tx-ops-nam-plc-adr>
      <tx-ops-nam-plc-lon>
        <tx-ops-nam-plc-lon-c1>0</tx-ops-nam-plc-lon-c1>
        <tx-ops-nam-plc-lon-c2>-71</tx-ops-nam-plc-lon-c2>
        <tx-ops-nam-plc-lon-c3>-2588</tx-ops-nam-plc-lon-c3>
        <tx-ops-nam-plc-lon-c4>-5000</tx-ops-nam-plc-lon-c4>
      </tx-ops-nam-plc-lon>
      <tx-ops-nam-plc-lat>
        <tx-ops-nam-plc-lat-c1>0</tx-ops-nam-plc-lat-c1>
        <tx-ops-nam-plc-lat-c2>42</tx-ops-nam-plc-lat-c2>
        <tx-ops-nam-plc-lat-c3>5454</tx-ops-nam-plc-lat-c3>
        <tx-ops-nam-plc-lat-c4>7000</tx-ops-nam-plc-lat-c4>
      </tx-ops-nam-plc-lat>
      <tx-ops-nam-plc-rad>
        <tx-ops-nam-plc-rad-c1>4</tx-ops-nam-plc-rad-c1>
        <tx-ops-nam-plc-rad-c2>0</tx-ops-nam-plc-rad-c2>
      </tx-ops-nam-plc-rad>
      <tx-ops-nam-cls>member</tx-ops-nam-cls>
      <tx-ops-nam-mbr-cnt>1</tx-ops-nam-mbr-cnt>
      <tx-ops-nam-mbr-brn>vectavi</tx-ops-nam-mbr-brn>
      <tx-ops-nam-mbr-ttl>support</tx-ops-nam-mbr-ttl>
      <tx-ops-nam-mbr-nms>
        <tx-ops-nam-mbr-nms-nam-cls>declared</tx-ops-nam-mbr-nms-nam-cls>
        <tx-ops-nam-mbr-nms-nam-als>badfood</tx-ops-nam-mbr-nms-nam-als>
      </tx-ops-nam-mbr-nms>
    </tx-ops>
  </tx>
</document>`

	var documentAsString string = common_preamble + docno1 + common_rest
	var documentAsString2 string = common_preamble + docno2 + common_rest

	//fmt.Printf("testInfoSpaceLifecycle: add document as string, should be a duplicate and return docId of -1\n")
	r_docId, err = is.InfoSpaceAddString(documentAsString)
	if DocIDToInt(r_docId) != -1 || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: with InfoSpaceAddString r_docId %v err %v\n", DocIDToInt(r_docId), err)
		return
	}

	//fmt.Printf("testInfoSpaceLifecycle: add document as string, should return a docId of >0\n")
	r_docId, err = is.InfoSpaceAddString(documentAsString2)
	if DocIDToInt(r_docId) <= 0 || err != nil {
		err = fmt.Errorf("testInfoSpaceLifecycle: with InfoSpaceAddString r_docId %v err %v\n", DocIDToInt(r_docId), err)
		return
	}

	fp2, err := filepath.Abs("data/raft.pdf")
	if err != nil {
		return
	}

	fp3, err := filepath.Abs("data/sitemap.xml")
	if err != nil {
		return
	}

	fp4, err := filepath.Abs("data/README-Why.txt")
	if err != nil {
		return
	}

	fp12, err := filepath.Abs("data/txdoc2.html")
	if err != nil {
		return
	}
	fp13, err := filepath.Abs("data/txdoc3.html")
	if err != nil {
		return
	}
	fp14, err := filepath.Abs("data/txdoc4.html")
	if err != nil {
		return
	}
	fp15, err := filepath.Abs("data/txdoc5.html")
	if err != nil {
		return
	}
	fp16, err := filepath.Abs("data/txdoc6.html")
	if err != nil {
		return
	}
	fp17, err := filepath.Abs("data/txdoc7.html")
	if err != nil {
		return
	}
	fp18, err := filepath.Abs("data/txdoc8.html")
	if err != nil {
		return
	}
	fp19, err := filepath.Abs("data/txdoc9.html")
	if err != nil {
		return
	}
	fp110, err := filepath.Abs("data/txdoc10.html")
	if err != nil {
		return
	}

	r_docId2, err := is.InfoSpaceAddFile(fp2)
	if DocIDToInt(r_docId2) != -1 || err != nil {
		err = fmt.Errorf("testAddFileClassType: with InfoSpaceAddFile r_docId2 %v err %v\n", DocIDToInt(r_docId2), err)
		return
	}

	r_docId3, err := is.InfoSpaceAddFile(fp3)
	if DocIDToInt(r_docId3) != -1 || err != nil {
		err = fmt.Errorf("testAddFileClassType: with InfoSpaceAddFile r_docId3 %v err %v\n", DocIDToInt(r_docId3), err)
		return
	}

	r_docId4, err := is.InfoSpaceAddFile(fp4)
	if DocIDToInt(r_docId4) != -1 || err != nil {
		err = fmt.Errorf("testAddFileClassType: with InfoSpaceAddFile r_docId4 %v err %v\n", DocIDToInt(r_docId4), err)
		return
	}

	r_docId12, err := is.InfoSpaceAddFile(fp12)
	if DocIDToInt(r_docId12) != -1 || err != nil {
		err = fmt.Errorf("testAddFileClassType: with InfoSpaceAddFile r_docId12 %v err %v\n", DocIDToInt(r_docId12), err)
		return
	}
	r_docId13, err := is.InfoSpaceAddFile(fp13)
	if DocIDToInt(r_docId13) != -1 || err != nil {
		err = fmt.Errorf("testAddFileClassType: with InfoSpaceAddFile r_docId13 %v err %v\n", DocIDToInt(r_docId13), err)
		return
	}
	r_docId14, err := is.InfoSpaceAddFile(fp14)
	if DocIDToInt(r_docId14) != -1 || err != nil {
		err = fmt.Errorf("testAddFileClassType: with InfoSpaceAddFile r_docId14 %v err %v\n", DocIDToInt(r_docId14), err)
		return
	}
	r_docId15, err := is.InfoSpaceAddFile(fp15)
	if DocIDToInt(r_docId15) != -1 || err != nil {
		err = fmt.Errorf("testAddFileClassType: with InfoSpaceAddFile r_docId15 %v err %v\n", DocIDToInt(r_docId15), err)
		return
	}
	r_docId16, err := is.InfoSpaceAddFile(fp16)
	if DocIDToInt(r_docId16) != -1 || err != nil {
		err = fmt.Errorf("testAddFileClassType: with InfoSpaceAddFile r_docId16 %v err %v\n", DocIDToInt(r_docId16), err)
		return
	}
	r_docId17, err := is.InfoSpaceAddFile(fp17)
	if DocIDToInt(r_docId17) != -1 || err != nil {
		err = fmt.Errorf("testAddFileClassType: with InfoSpaceAddFile r_docId17 %v err %v\n", DocIDToInt(r_docId17), err)
		return
	}
	r_docId18, err := is.InfoSpaceAddFile(fp18)
	if DocIDToInt(r_docId18) != -1 || err != nil {
		err = fmt.Errorf("testAddFileClassType: with InfoSpaceAddFile r_docId18 %v err %v\n", DocIDToInt(r_docId18), err)
		return
	}
	r_docId19, err := is.InfoSpaceAddFile(fp19)
	if DocIDToInt(r_docId19) != -1 || err != nil {
		err = fmt.Errorf("testAddFileClassType: with InfoSpaceAddFile r_docId19 %v err %v\n", DocIDToInt(r_docId19), err)
		return
	}
	r_docId110, err := is.InfoSpaceAddFile(fp110)
	// data/params.xml set maxdocs = 11, this is the 12th add so should fail with "limit reached"
	if DocIDToInt(r_docId110) != -2 || err != nil {
		err = fmt.Errorf("testAddFileClassType: with InfoSpaceAddFile r_docId110 %v err %v\n", DocIDToInt(r_docId110), err)
		return
	}

	fmt.Printf("testAddFileClassType: completed with infospace %v err %v\n", is, err)

	return

}
